import range
from gnuplot import Gnuplot
import window

class Axes(window.Window):
	def __init__(self):
		window.Window.__init__(self, "axes")
		self.set_ranges()
	
	def set_ranges(self):
		
		ranges = ['x', 'y', 'z', 'x2', 'y2', 't', 'r', 'u', 'v', 'cb']
		for ran in ranges:
			self.set_range(ran)
	
	def set_range(self, ran):
		r = range.Range(ran)
		if not r.is_default:
			self.child("txt_%s_inf" % ran).set_text(str(r.lower))
			self.child("txt_%s_sup" % ran).set_text(str(r.upper))
			self.child("tb_%s_rev" % ran).set_active(r.is_reverse)
			self.child("tb_%s_wb" % ran).set_active(r.is_writeback)

	def on_cmd_cancel_clicked(self, widget, data = None):
		self.window.destroy()

	def on_cmd_apply_clicked(self, widget, data = None):
		gp = Gnuplot()
		
		ranges = ['x', 'y', 'z', 'x2', 'y2', 't', 'r', 'u', 'v', 'cb']
		for ran in ranges:
			l = self.child("txt_%s_inf" % ran).get_text()
			u = self.child("txt_%s_sup" % ran).get_text()
			if l == "" and u == "": continue
			r = range.Range(ran)
			r.lower = l
			r.upper = u
			r.is_reverse = self.child("tb_%s_rev" % ran).get_active()
			r.is_writeback = self.child("tb_%s_wb" % ran).get_active()
			gp.set_range(ran, r)
		
		if gp.notify_last(self.window): return
		gp.replot()
			
	def on_cmd_ok_clicked(self, widget, data = None):
		self.on_cmd_apply_clicked(widget, data)
		self.on_cmd_cancel_clicked(widget, data)
		
	def on_cmd_restore_clicked(self, widget, data = None):
		gp = Gnuplot()
		ran = widget.get_text()
		
		gp.set_range(ran, range.RESTORE)
		self.set_range(ran)
