import gtk
import glib
import os
import pipe
import ptty
import re

PIPE = True

TIMEOUT = 40

ERROR = gtk.MESSAGE_ERROR
INFO = gtk.MESSAGE_INFO

class Gnuplot:
	__shared_state = {}
	def __init__(self):
		self.__dict__ = self.__shared_state

	def start(self):
		if PIPE == False:
			self.pipe = ptty.Ptty(100)
		else:
			self.pipe = pipe.Pipe("gnuplot", timeout = TIMEOUT)

	def stop(self):
		self.pipe.close()

	def answer(self):
		ans = ""
		while True:
			ans = ans + self.pipe.readlines()
			ans = ans.replace(getattr(self,"last_cmd",""), "", 1)
			ans = ans.replace("gnuplot> ", "         ")
			ans = ans.replace("line 0: ", "        ")
			ans = ans.replace("\n         ", "\n")
			ans = ans.replace("\n\n", "\n")
			if ans.find("Press return for more:") is -1: break
			ans = ans.replace("Press return for more:", "")
			self.pipe.write("\n")

		self.pipe.write("\n\n\n")

		if ans == "" or ans == None or len(ans) < 4:
			return None

		if ans.find('^') != -1 or ans.find("line 0: ") != -1:
			if ans.find(self.last_cmd) == -1:
				ans = "\n%s%s" % (self.last_cmd, ans)
			self.last_error = ans
			return ERROR

		self.last_info = ans
		return INFO

	def cd(self, new_dir):
		return self.cmd("cd '%s'" % new_dir)

	def clear(self):
		return self.cmd("clear")

	def help(self, voice):
		return self.cmd("help %s" % voice)

	def plot(self, func, ran = ""):
		return self.cmd("plot %s %s" % (ran, func))

	def splot(self, func, ran = ""):
		return self.cmd("splot %s %s" % (ran, func))

	def cmd(self, string):
		self.last_cmd = "%s" % string
		self.pipe.write("%s\n" % string)
		ans = self.answer()
		self.last_event = ans

		return ans

	def get_last_event(self):
		evt = getattr(self, "last_event", None)
		if evt == None:	return None
		if evt == ERROR: return self.last_error
		return self.last_info

	def load(self, filename):
		return self.cmd("load '%s'" % filename)

	def notify(self, parent, cmd):
		self.cmd(cmd)
		return self.notify_last(parent)

	def notify_last(self, parent):
		if self.last_event is not None:
			dlg = gtk.MessageDialog(parent, gtk.DIALOG_MODAL, self.last_event, gtk.BUTTONS_OK)
			if self.last_event == ERROR:
				dlg.set_markup('Gnuplot encountered the following error:')
				dlg.format_secondary_markup('<tt>%s</tt>' % glib.markup_escape_text(self.last_error).strip())
			else:
				dlg.set_markup('Gnuplot informs you that:')
				dlg.format_secondary_markup('<tt>%s</tt>' % glib.markup_escape_text(self.last_info).strip())
			dlg.run()
			dlg.destroy()

		return self.last_event

	def pwd(self):
		return self.cmd("pwd")

	def replot(self):
		return self.cmd("replot")

	def save(self, what = "", where = "-"):
		return self.cmd("save %s '%s'" % (what, where))
	
	def set_margin(self, which, value, screen = False):
		return self.cmd("set %smargin %s%s" % (which[0].lower(), "at screen " if screen and value >= 0 else "", "" if value < 0 else str(value)))

	def set_multiplot(self, args = ""):
		return self.cmd("set multiplot %s" % args)

	def set_origin(self, x, y, coord = ""):
		return self.cmd("set origin %s%s,%s" % (coord, x, y))

	def set_output(self, filename = ''):
		return self.cmd("set output '%s'" % filename)

	def set_parametric(self):
		return self.cmd("set parametric")

	def set_range(self, axis, ran):
		return self.cmd("set %srange %s" % (axis, ran))

	def set_size(self, x, y, ratio = None):
		return self.cmd("set size %s%s,%s" % ("ratio %s " % ratio if ratio else "noratio ", x, y))

	def set_terminal(self, term, args = ""):
		return self.cmd("set terminal %s %s" % (term, args))

	def show(self, what):
		return self.cmd("show %s" % what)

	def unset(self, what):
		return self.cmd("unset %s" % what)
