import gtk
from gnuplot import Gnuplot
import dialog

class Multiplot(dialog.Dialog):
	def __init__(self):
		dialog.Dialog.__init__(self, "multiplot")
		
		self.window.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK)
		
		self.child("adj_rows").set_value(1)
		self.child("adj_cols").set_value(1)

	def get_layout(self):
		i = self.child("cbo_layout").get_active()
		if i == 0:
			return self.make_grid(1, 2)
		elif i == 1:
			return self.make_grid(2, 1)
		elif i == 2:
			return self.make_grid(2, 2)
		elif i == 3:
			return self.make_grid(4, 4)
		elif i == 4:
			if self.child("rdo_grid").get_active():
				return self.make_grid(self.child("spb_rows").get_value_as_int(),self.child("spb_cols").get_value_as_int())
			else:
				return self.make_custom_layout()		
	
	def draw_layout(self, layouts):
		dwg = self.child("dwg_layout")
		canvas = dwg.window
		try:
			w, h = canvas.get_size()
		except AttributeError:
			return
			
		black = dwg.get_style().black_gc
		white = dwg.get_style().white_gc
		
		canvas.draw_rectangle(white, True, 0, 0, w, h)
		#canvas.draw_rectangle(black, False, 0, 0, w - 1, h - 1)
		w = w - 1
		h = h - 1
		
		for layout in layouts:
			try:
				canvas.draw_rectangle(black, False, int(float(layout[0]) * w) ,\
													int((1 - float(layout[1])) * h - float(layout[3]) * h),\
													int(float(layout[2]) * w),\
													int(float(layout[3]) * h))
			except ValueError:
				pass

	def make_grid(self, rows, cols):
		t = self.child("spb_top").get_value()
		b = self.child("spb_bottom").get_value()
		l = self.child("spb_left").get_value()
		r = self.child("spb_right").get_value()
		
		h = self.child("spb_hor").get_value()
		v = self.child("spb_ver").get_value()
		
		layouts = []
		
		wi = (1. - (l + r) - h * (cols - 1)) / cols
		he = (1. - (t + b) - v * (rows - 1)) / rows
		
		y = 1 - t - he
		for row in range(rows):
			x = l
			for col in range(cols):
				layouts.append((x, y, wi, he))
				x = x + wi + h
			y = y - he - v
		
		return layouts
	
	def draw_grid(self, rows, cols):
		self.draw_layout(self.make_grid(rows, cols))

	def update_preview(self):
		i = self.child("cbo_layout").get_active()
		if i == 0:
			self.draw_grid(1, 2)
		elif i == 1:
			self.draw_grid(2, 1)
		elif i == 2:
			self.draw_grid(2, 2)
		elif i == 3:
			self.draw_grid(4, 4)
		elif i == 4:
			if self.child("rdo_grid").get_active():
				self.draw_grid(self.child("spb_rows").get_value_as_int(),self.child("spb_cols").get_value_as_int())
			else:
				self.draw_custom_layout()
				
	def make_custom_layout(self):
		tb = self.child("tb_custom")
		data = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).splitlines()
		layouts = []
		for line in data:
			a = line.split(',')
			if len(a) == 4: layouts.append(a)
		return layouts
	
	def draw_custom_layout(self):
		self.draw_layout(self.make_custom_layout())
				

	def on_cbo_layout_changed(self, widget, data = None):
		i = widget.get_active()
		self.child("tbl_custom").set_sensitive(i == 4)
		self.update_preview()
		
	def on_rdo_custom_toggled(self, widget, data = None):
		status = widget.get_sensitive()
		self.sensitiveness(["hb_grid", "fr_margins", "fr_spacing"], not status)
		self.sensitiveness(["tv_custom"], status)

	def on_rdo_grid_toggled(self, widget, data = None):
		status = widget.get_sensitive()
		self.sensitiveness(["hb_grid", "fr_margins", "fr_spacing"], status)
		self.sensitiveness(["tv_custom"], not status)

	def on_spb_value_changed(self, widget, data = None):
		self.update_preview()

	def on_dwg_layout_expose_event(self, widget, event, data = None):
		self.update_preview()
		return False

	def on_txv_custom_key_press_event(self, widget, event, data = None):
		self.draw_custom_layout()
