import re
from gnuplot import Gnuplot

class Plot:
	def __init__(self):
		self.title = self.function = self.is_datafile = self.style = self.ls = \
		self.lt = self.lw = self.pt = self.ps = self.fs = self.is_palette = self.index = \
		self.every = self.using = self.smooth = self.is_surface = self.is_ls = \
		self.is_parametric = self.fs_index = self.style_index = self.smooth_index = \
		self.is_matrix = self.matrix = self.matrix_index = self.lc = None

	def __repr__(self):
		format = ""
		if self.is_datafile:
			if self.is_matrix: format = "%s matrix%s" % (format, (" %s" % self.matrix.lstrip()).rstrip())
			if self.index:  format = "%s index %s"  % (format, str(self.index))
			if self.using:  format = "%s using %s"  % (format, str(self.using))
			if self.every:  format = "%s every %s"  % (format, str(self.every))
			if self.smooth: format = "%s smooth %s" % (format, str(self.smooth))

		if self.style:
			format = "%s with %s" % (format, self.style)
			if self.is_ls: format = "%s ls %s" % (format, self.ls)
			else:
				if self.lt: format = "%s lt %s" % (format, self.lt)
				if self.lw: format = "%s lw %s" % (format, self.lw)
				if self.pt: format = "%s pt %s" % (format, self.pt)
				if self.ps: format = "%s ps %s" % (format, self.ps)

				if self.fs:         format = "%s fs %s" % (format, self.fs)
				if self.is_palette: format = "%s palette" % format
				
		if self.lc: format = "%s lc %s" % (format, self.lc)

		if self.title: format = r"""%s title '%s'""" % (format, self.title)
		
		return "%s %s" % (self.function, format.strip())

	def parse(self, format):
		tokens = [("index", "(\d+)"), ("us?i?n?g?", "(.+?)"), ("every", "(.+?)"), ("smooth", "(.+?)"), \
		          ("title", r"""["'](.+?)["']"""), ("wi?t?h?", "(.+?)"), ("ls", "(\d+)"), ("lt", "(\d+)"), ("lw", "(\d+)"), \
		          ("pt", "(\d+)"), ("ps", "(\d+)"), ("fs", "(.+?)"), ("palette", 0), ("matrix", "(.*?)"), ("matrix", 0),\
		          ("lc", "(.+?)")]
		styles = ["", "lines", "points", "linespoints", "impulses", "dots", "steps", \
		          "fsteps", "histeps", "errorbars", "labels", "xerrorbars", "yerrorbars", \
		          "xyerrorbars", "errorlines", "xerrorlines", "yerrorlines", "xyerrorlines", \
		          "boxes", "histograms", "filledcurves", "boxerrorbars", "boxxyerrorbars", \
		          "financebars", "candlesticks", "vectors", "image", "rgbimage", "pm3d"]

		alltokens = "\n"
		for token in tokens:
			alltokens = "%s|%s" % (alltokens, token[0])

		# Determine the function
		match = re.search("(.+?) (?:%s)" % alltokens, format.strip() + " \n", re.DOTALL)
		if not match: return None
		self.function = match.group(1)

		self.is_datafile = True if self.function and re.match('"(.+)"', self.function.strip()) else False
		self.is_ls = False
		# Maybe this can be improved by capturing the next token and using it for
		# the next search until \n is reached.
		for token in tokens:
			if token[1]:
				pat = "%s %s[ ]?(?:%s)" % (token[0], token[1], alltokens)
				match = re.search(pat, format.strip() + " \n", re.DOTALL)
				if match:
					exec('self.%s = "%s"' % (token[0].replace("?","").replace("with", "style"), match.group(1)))
					if token[0] == 'ls': self.is_ls = True
			else:
				match = format.find(token[0])
				exec("self.is_%s = True if match != -1 else False" % token[0])

		if self.style:
			for style in styles:
				if style.find(self.style.strip()) == 0:
					self.style_index = styles.index(style)
					break

		if not self.title: self.title = self.function

	def test(self, parent):
		gp = Gnuplot()
		gp.set_terminal("push")
		gp.set_terminal("unknown")
		ans = gp.plot(repr(self)) if not self.is_surface else gp.splot(repr(self))
		gp.notify_last(parent)
		gp.set_terminal("pop")

		return ans
