from gnuplot import Gnuplot
import re
import gtk
import glib
import dialog
import os.path
import gnuplot
import tempfile
import subprocess as subp

class Render(dialog.Dialog):
	def __init__(self):
		dialog.Dialog.__init__(self, "render")

		self.window.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		self.window.add_button("Render", gtk.RESPONSE_ACCEPT)
		self.child("spb_res").set_value(128)

		gp = Gnuplot()

		gp.set_terminal("")

		ls = self.child("ls_terminal")
		terms = re.findall("[ ]*(.+?)  (.+)", gp.get_last_event())
		for term in terms:
			if term[0] == "epslatex": return

		self.dlg_error("Your installation of Gnuplot does not provide the 'epslatex' terminal.")
		self.window.response(gtk.RESPONSE_NONE)

	def render_current_view(self):
		filename = self.window.get_filename()
		preamble_file = self.child("fc_preamble").get_filename()

		tmp = tempfile.gettempdir()

		gp = Gnuplot()
		gp.set_terminal("push")
		gp.set_terminal("epslatex", "color")
		gp.set_output("%s/plot.tex" % tmp)
		gp.cmd("replot;set o") # This is necessary, otherwise the output would be incomplete
		gp.set_terminal("pop")

		if preamble_file:
			f = open(preamble_file, 'r')
			preamble = '%s\n\\begin{document}\\input{%s/plot.tex}\\end{document}' % (f.read(), tmp)
			f.close()
		else:
			preamble = r'\documentclass{minimal}\usepackage{graphicx,color}\begin{document}\input{%s/plot.tex}\end{document}' % tmp

		latex_file = open('%s/latex.tex' % tmp, 'w')
		latex_file.write(preamble)
		latex_file.close()

		self.shell_exec('latex', "latex -halt-on-error -output-directory %s %s/latex.tex" % (tmp, tmp))

		root, ext = os.path.splitext(filename)
		choice = self.child("cbo_format").get_active()
		
		if choice == 0: # png
			if ext != ".png": filename = filename + ".png"
			self.dvipng(filename, tmp)
			
		elif choice == 1: # ps
			if ext != ".ps": filename = filename + ".ps"
			if self.dvips(filename, tmp): return
			
		elif choice == 2: # pdf
			if ext != ".pdf": filename = filename + ".pdf"
			if self.dvips("%s/latex.ps" % tmp, tmp): return
			if self.ps2pdf(filename, tmp): return
			
		else:
			self.dlg_error("Unrecognized action.")

		return 0

	def dvipng(self, filename, tmp):
		res = self.child("spb_res").get_value_as_int()
		is_transparent = self.child("chk_transparency").get_active()
		exec_line = "dvipng -q* -T tight -z 9 %s -D %d -o %s %s/latex.dvi" % \
				   ("-bg Transparent" if is_transparent else "", res, filename, tmp)
		
		return self.shell_exec('dvipng', exec_line)

	def dvips(self, filename, tmp):
		return self.shell_exec('dvips', "dvips -f -E -q -o %s %s/latex.dvi" % (filename, tmp))

	def ps2pdf(self, filename, tmp):
		return self.shell_exec('ps2pdf', "ps2pdf %s/latex.ps %s" % (tmp, filename))
		
	def shell_exec(self, app, exec_line):
		try:
			shell = subp.Popen(exec_line.split(), stdout = subp.PIPE)
		except OSError:
			self.dlg_error("The command '%s' has failed to execute. Make sure that it is correctly installed on your machine." % app)
			return 1

		if shell.wait():
			self.dlg_error("The external application '%s' encountered an error while executing." % app)
			return 1

		return 0
