import shelve
from gnuplot import Gnuplot

class Session:
	__shared_state = {
		'plot_uid' : 0,
		'plot_dict' : {},
		'history' : [],
		'has_changed' : False,
		'filename' : None,
		'is_multiplot' : False,
		'multiplot_data' : None,
		'multiplot_count' : 0,
		'multiplot_total' : 0
	}
	
	def __init__(self):
		self.__dict__ = self.__shared_state

	def clear(self):
		self.plot_uid = 0
		self.plot_dict = {}
		self.history = []
		self.has_changed = True

	def new(self):
		self.plot_uid = 0
		self.plot_dict = {}
		self.history = []
		self.has_changed = False
		self.filename = None

	def save(self, fn = None):
		if fn:
			filename = fn
			self.has_changed = True
		else: filename = self.filename
		if not filename: return False
		if not self.has_changed:
			self.status("No changes need to be saved.")
			return False

		try:
			gp = Gnuplot()
			shelf = shelve.open(filename, 'n')
			shelf['ts_plot'] = self.plot_hierarchy()
			shelf['plot_dict'] = self.plot_dict
			shelf['plot_uid'] = self.plot_uid
			shelf['state'] = gp.get_last_event() if gp.save() is not None else ""
			shelf['history'] = self.history
			shelf.close()
			self.filename = filename
			self.status('File "%s" has been saved successfully.' % self.filename)
		except IOError:
			self.status('Unable to save "%s".' % self.filename)
			return False
			
		self.has_changed = False
		return True
		
	def open(self, filename):
		gp = Gnuplot()

		shelf = shelve.open(filename)
		try:
			model = shelf['ts_plot']
			if len(model):
				if len(model[0]):
					if len(model[0][0])!= 3: raise ValueError
			self.ts_plot = model
			self.plot_dict = shelf['plot_dict']
			self.plot_uid = shelf['plot_uid']
			self.state = shelf['state'] if 'state' in shelf else ""
			self.history = shelf['history'] if 'history' in shelf else []
			
			self.filename = filename
			self.has_changed = False
			
		except (KeyError, ValueError):
			return False

		finally:
			shelf.close()
			self.status('File "%s" has been loaded successfully.' % self.filename)

		return True
